package edu.nyu.scps.bed;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.InputType;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //Insert your Java code between the two horizontal lines.
        //-----------------------------
        for (;;) {
            float miles = getFloat("Miles", "How many miles did you drive?");
            float gallons = getFloat("Gallons", "How many gallons did you use?");

            float mpg;
            if (gallons == 0) {
                mpg = Float.POSITIVE_INFINITY;
            } else {
                mpg = miles / gallons;
            }

            display("MPG", "Miles per gallon = " + mpg);
        }
        //-----------------------------
    }

    //The handleMessage method of this object will be called when we call the sendMessage method of
    //this object.  It throws an exception to break us out of the infinite loops below.
    //When called from get, it also carries the String that the user typed in.

    private static class MyHandler extends Handler {
        @Override
        public void handleMessage(Message message) {
            if (message.obj == null) {
                throw new RuntimeException();
            }
            throw new RuntimeException(message.obj.toString());
        }
    }

    private MyHandler handler = new MyHandler();

    private String get(String title, String message, int inputType) {
        //A builder object can create a dialog object.
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title);
        builder.setMessage(message);

        //This inflater reads the dialog.xml and creates the objects described therein.
        //Pass null as the parent view because it's going in the dialog layout.
        LayoutInflater inflater = getLayoutInflater();
        View view = inflater.inflate(R.layout.dialog, null);
        EditText editText = (EditText)view.findViewById(R.id.editText);
        editText.setInputType(inputType);
        builder.setView(view);

        editText.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (event.getAction() == KeyEvent.ACTION_DOWN
                        && keyCode == KeyEvent.KEYCODE_ENTER) {
                    EditText editText = (EditText)v;
                    Editable editable = editText.getText();
                    //Sending this message will break us out of the loop below.
                    Message message = handler.obtainMessage();
                    message.obj = editable.toString();
                    handler.sendMessage(message);
                }
                return false;
            }
        });

        AlertDialog alertDialog = builder.create();
        alertDialog.show();

        //Loop until the user presses the EditText's Done button.
        String retVal = "";
        try {
            Looper.loop();
        } catch (RuntimeException runtimeException) {
            retVal = runtimeException.getMessage();
        }
        alertDialog.dismiss();
        return retVal;
    }

    private int getInt(String title, String message) {
        String s = get(title, message, InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_SIGNED);
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException numberFormatException) {
            Toast.makeText(MainActivity.this, "bad int " + s, Toast.LENGTH_LONG).show();
            return 0;
        }
    }

    private long getLong(String title, String message) {
        String s = get(title, message, InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_SIGNED);
        try {
            return Long.parseLong(s);
        } catch (NumberFormatException numberFormatException) {
            Toast.makeText(MainActivity.this, "bad long " + s, Toast.LENGTH_LONG).show();
            return 0L;
        }
    }

    private boolean getBoolean(String title, String message) {
        String s = get(title, message, InputType.TYPE_CLASS_TEXT);
        //Boolean.parseBoolean() does not throw any exception.
        if (s.equals("true")) {
            return true;
        }
        if (!s.equals("false")) {
            Toast.makeText(MainActivity.this, "bad bool " + s, Toast.LENGTH_LONG).show();
        }
        return false;
    }

    private float getFloat(String title, String message) {
        String s = get(title, message,
                InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_NUMBER_FLAG_DECIMAL);
        try {
            return Float.parseFloat(s);
        } catch (NumberFormatException numberFormatException) {
            Toast.makeText(MainActivity.this, "bad float " + s, Toast.LENGTH_LONG).show();
            return 0f;
        }
    }

    private double getDouble(String title, String message) {
        String s = get(title, message,
                InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_NUMBER_FLAG_DECIMAL);
        try {
            return Double.parseDouble(s);
        } catch (NumberFormatException numberFormatException) {
            Toast.makeText(MainActivity.this, "bad double " + s, Toast.LENGTH_LONG).show();
            return 0d;
        }
    }

    private String getString(String title, String message) {
        //No such thing as a bad String.
        return get(title, message, InputType.TYPE_CLASS_TEXT);
    }

    private void display(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                //Sending this message will break us out of the loop below.
                Message message = handler.obtainMessage();
                handler.sendMessage(message);
            }
        });

        AlertDialog alertDialog = builder.create();
        alertDialog.show();

        //Loop until the user presses the EditText's Done button.
        try {
            Looper.loop();
        } catch (RuntimeException runtimeException) {
        }
    }

    private void display(String title, int i) {
        display(title, Integer.toString(i));
    }

    private void display(String title, long l) {
        display(title, Long.toString(l));
    }

    private void display(String title, boolean b) {
        display(title, Boolean.toString(b));
    }

    private void display(String title, float f) {
        display(title, Float.toString(f));
    }

    public void display(String title, double d) {
        display(title, Double.toString(d));
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
